/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.databene.commons.Visitor;
import org.databene.commons.comparator.LongComparator;
import org.databene.commons.file.FilePrintStream;
import org.databene.dbsanity.model.Reportable;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.util.CheckVisitor;
import org.databene.html.HTMLUtil;

/**
 * Lists all checks with defects ordered by descending defect count.<br/><br/>
 * Created: 15.12.2010 09:48:25
 * @since 0.6
 * @author Volker Bergmann
 */
public class DefectsPerCheckModule extends AbstractReportModule {

	private List<SanityCheck> defectiveChecks;
	private long totalDefects;

	public DefectsPerCheckModule() {
		this.defectiveChecks = new ArrayList<SanityCheck>();
		this.totalDefects = 0;
	}

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return (scope == ReportScope.ROOT ? 2 : 0);
	}
	
	@Override
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, FilePrintStream out) {
		div(5, out);
	}
	
	@Override
	public void summary(SanityCheckSuite rootSuite) {
		scanSuite(rootSuite);
		createDetailsPage(rootSuite);
	}

	public String getFileName() {
		return "defects_per_check.html";
	}

	// private helpers -------------------------------------------------------------------------------------------------
	
	private void scanSuite(SanityCheckSuite suite) {
		// fetch all defective checks
		Visitor<Reportable> visitor = new CheckVisitor() {
			public void visitCheck(SanityCheck check) {
				if (check.getDefectCount() > 0) {
					defectiveChecks.add(check);
					totalDefects += check.getDefectCount();
				}
			}
		};
		suite.accept(visitor);
		
		// order checks by descending defect count
		Collections.sort(defectiveChecks, new Comparator<SanityCheck>() {
			public int compare(SanityCheck count1, SanityCheck count2) {
				return - LongComparator.compare(count1.getDefectCount(), count2.getDefectCount());
			}
		});
	}

	private void createDetailsPage(SanityCheckSuite rootSuite) {
		if (defectiveChecks.size() == 0)
			return;
		String title = "Defects per Check";
		FilePrintStream out = null;
		File file = new File(rootSuite.getReportFolder(), getFileName());
		try {
			out = openNewFile(file, title);
			out.println(context.navBar(rootSuite.getReportFolder()));
			div(-1, out);
		} catch (FileNotFoundException e) {
			throw new RuntimeException("Error writing defects per check file: " + file, e);
		} finally {
			super.closeFile(out);
		}
	}

	private void div(int limit, FilePrintStream out) {
		boolean halfReached = false;
		long defectsSoFar = 0;
		startModule(out);
		String title = (limit >= 0 ? "Defects per Check" : context.formatLong(defectiveChecks.size()) + " Checks with Defects");
		modHeader(title, out);
		if (defectiveChecks.size() > 0) {
			long maxCount = defectiveChecks.get(0).getDefectCount();
			// render HTML
			startModBody(out);
			out.println("	<table cellpadding='2px' width='100%'>");
			out.println("		<tr>");
			out.println("			<th align='left' width='30%'>Check</th>");
			out.println("			<th align='left' width='70%'>Count</th>");
			out.println("		</tr>");
			for (int i = 0; (i < limit || limit < 0) && i < defectiveChecks.size(); i++) {
				SanityCheck check = defectiveChecks.get(i);
				String percentage = String.valueOf(check.getDefectCount() * 100 / maxCount) + "%";
				out.println("		<tr>");
				out.println("			<td align='left'>" + ReportUtil.docOrDefectFileLink(check, out.getFile()) + "</td>");
				out.println("			<td align='left'>");
				ReportUtil.numberBar(check.getDefectCount(), percentage, "#" + ReportContext.RED, context, out);
				out.println("			</td align='left'>");
				out.println("		</tr>");
				defectsSoFar += check.getDefectCount();
				if (!halfReached // marker not yet painted 
						&& defectsSoFar > totalDefects / 2 // 50% or more 
						&& defectsSoFar < totalDefects) { // if it is the only check, we don't need the marker
					printHalfVolumeRuler(defectsSoFar * 100 / totalDefects, out);
					halfReached = true;
				}
			}
			// if we're currently rendering a dashboard view and there are more elements
			if (limit >= 0 && limit < defectiveChecks.size())
				renderDetailsLink(out);
			out.println("	</table>");
			endModBody(out);
		} else {
			out.println("none");
		}
		endModule(out);
	}

	private void printHalfVolumeRuler(long percentage, FilePrintStream out) {
		out.println("		<tr>");
		out.println("			<td colspan='2'><table width='100%'><tr><td width='0%'><font size='-1' color='#aaa'>" + percentage + "%</font>&nbsp;</td><td width='100%'><hr/></td></tr></table></td>");
		out.println("		</tr>");
	}

	public void renderDetailsLink(FilePrintStream out) {
		out.println("		<tr>");
		out.println("			<td align='left'>" + HTMLUtil.a(getFileName(), "[more...]") + "</td>");
		out.println("			<td>...</td>");
		out.println("		</tr>");
	}

}
