/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.databene.commons.collection.OrderedNameMap;
import org.databene.commons.file.FilePrintStream;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.util.CheckTagAggregator;
import org.databene.dbsanity.util.TagCloudUtil;
import org.databene.html.HTMLUtil;

/**
 * Lists the number of defects per tag.<br/><br/>
 * Created: 11.11.2011 17:18:19
 * @since 0.9.2
 * @author Volker Bergmann
 */
public class DefectsPerTagModule extends AbstractReportModule {
	
	private static final int DASHBOARD_ENTRIES = 10;

	private static final String DEFECT_COLOR = "#AA0000";
	private static final int MIN_FONT_SIZE = 80;
	private static final int MAX_FONT_SIZE = 200;

	private Map<String, Long> weights;
	
	public DefectsPerTagModule() {
	}

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return (scope == ReportScope.ROOT ? 1 : 0);
	}
	
	@Override
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, FilePrintStream out) {
		div(DASHBOARD_ENTRIES, out);
	}
	
	@Override
	public void summary(SanityCheckSuite rootSuite) {
		scanSuite(rootSuite);
		createDetailsPage(rootSuite);
	}

	private void scanSuite(SanityCheckSuite suite) {
		CheckTagAggregator visitor = new CheckTagAggregator(
				context.isUsingNamespaces(), context.isAuthorInCloud(), context.isTableInCloud()) {
			@Override
			protected long weight(SanityCheck check) {
				return check.getDefectCount();
			}
		};
		suite.accept(visitor);
		weights = visitor.getWeights();
	}

	private void createDetailsPage(SanityCheckSuite rootSuite) {
		if (weights.size() == 0)
			return;
		String title = "Defects per Tag";
		FilePrintStream out = null;
		File file = new File(rootSuite.getReportFolder(), "defects_per_tag.html");
		try {
			out = openNewFile(file, title);
			out.println(context.navBar(rootSuite.getReportFolder()));
			div(-1, out);
		} catch (FileNotFoundException e) {
			throw new RuntimeException("Error writing defect type count file: " + file, e);
		} finally {
			super.closeFile(out);
		}
	}

	private void div(int limit, FilePrintStream out) {
		// order defect type counts in descending order
		List<Entry<String, Long>> counts = new ArrayList<Map.Entry<String, Long>>(weights.entrySet());
		Collections.sort(counts, new Comparator<Map.Entry<String, Long>>() {
			public int compare(Entry<String, Long> count1, Entry<String, Long> count2) {
				return - count1.getValue().compareTo(count2.getValue());
			}
		});
		startModule(out);
		String title = (limit >= 0 ? "Defects per Tag" : context.formatLong(weights.size()) + " Tags with Defects");
		modHeader(title, out);
		if (counts.size() > 0) {
			startModBody(out);
			renderTagCloud(counts, limit, out);
			if (limit <= 0) {
				out.println("<br/>");
				out.println("<br/>");
				renderBars(limit, out, counts);
			}
			endModBody(out);
		} else {
			out.println("none");
		}
		endModule(out);
	}

	private void renderTagCloud(List<Entry<String, Long>> counts, int limit, FilePrintStream out) {
		int n = counts.size();
		if (limit > 0)
			n = Math.min(n, limit); 
		Map<String, Long> usedCounts = OrderedNameMap.<Long>createCaseInsensitiveMap();
		for (int i = 0; i < n; i++) {
			Entry<String, Long> entry = counts.get(i);
			usedCounts.put(entry.getKey(), entry.getValue());
		}
		TagCloudUtil.printTagCloud(usedCounts, DEFECT_COLOR, MIN_FONT_SIZE, MAX_FONT_SIZE, out);
		if (limit > 0) {
			out.print(" ");
			out.print(HTMLUtil.a("defects_per_tag.html", "<span style='font-size:" + MIN_FONT_SIZE + "%'>[more...]</span>"));
		}
	}

	protected void renderBars(int limit, FilePrintStream out,
			List<Entry<String, Long>> counts) {
		long maxCount = counts.get(0).getValue();
		// render HTML
		out.println("	<table width='100%' cellpadding='2px'>");
		out.println("		<tr>");
		out.println("			<th align='left' width='20%'>Tag</th>");
		out.println("			<th align='left' width='80%'>Defect Count</th>");
		out.println("		</tr>");
		for (int i = 0; (i < limit || limit < 0) && i < counts.size(); i++) {
			Entry<String, Long> count = counts.get(i);
			String percentage = String.valueOf(count.getValue() * 100 / maxCount) + "%";
			out.println("		<tr>");
			out.println("			<td align='left'>" + (count.getKey() != null ? count.getKey() : "unknown") + "</td>");
			out.println("			<td align='left'>");
			ReportUtil.numberBar(count.getValue(), percentage, DEFECT_COLOR, context, out);
			out.println("			</td>");
			out.println("		</tr>");
		}
		if (limit >= 0 && limit < counts.size()) {
			out.println("		<tr>");
			out.println("			<td align='left'>" + HTMLUtil.a("defects_per_tag.html", "[more...]") + "</td>");
			out.println("			<td></td>");
			out.println("		</tr>");
		}
		out.println("	</table>");
	}

}
