/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.util.Map;

import org.databene.commons.ArrayFormat;
import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.TimeUtil;
import org.databene.commons.file.FilePrintStream;
import org.databene.commons.version.VersionInfo;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.util.CheckTagAggregator;
import org.databene.jdbacl.JDBCConnectData;

/**
 * {@link ReportModule} that provides general information about database and 
 * report execution.<br/><br/>
 * Created: 08.11.2010 17:10:54
 * @since 1.0
 * @author Volker Bergmann
 */
public class InfoModule extends AbstractReportModule {

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return 1;
	}
	
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, FilePrintStream out) {
        JDBCConnectData connectData = (JDBCConnectData) context.get("connectData");
	    startModule(out);
	    renderInfo(connectData, out);
        renderStats(suite, out);
        renderCheckedTags(suite, out);
        endModBody(out);
		endModule(out);
    }

	protected void renderInfo(JDBCConnectData connectData, FilePrintStream out) {
		modHeader("Settings", out);
	    startModBody(out);
        out.println("			<table cellpadding='3'>");
        // DB Sanity version
        printInfoRow("DB Sanity Version",  VersionInfo.getInfo("dbsanity").getVersion(), out);
        // app version
        Object appVersion = context.getAppVersion();
        if (appVersion != null && !"*".equals(appVersion.toString().trim()))
            printInfoRow("Application Version", appVersion.toString(), out);
        // URL
        printInfoRow("URL",  connectData.url, out);
        // User
        printInfoRow("User", connectData.user, out);
        // Table restriction
        if (!CollectionUtil.isEmpty(context.getTablesToCheck()))
        	printInfoRow("Table Restriction", ArrayFormat.format(context.getTablesToCheck().toArray()), out);
        // Tag restriction
        if (!CollectionUtil.isEmpty(context.getTags()))
        	printInfoRow("Tag Restriction", ArrayFormat.format(context.getTags().toArray()), out);
        // Defect count restriction
        if (context.getDefectCountLimit() > 0)
        	printInfoRow("Defect Count Limit", String.valueOf(context.getDefectCountLimit()), out);
        if (context.getDefectRowLimit() > 0)
        	printInfoRow("Defect Row Limit", String.valueOf(context.getDefectRowLimit()), out);
        // Ignorable tables
        printInfoRow("Ignorable Tables", renderBreakableRegex(context.getIgnorableTables()), out);
        // condensed mode
        if (context.isCondensed())
        	printInfoRow("Mode", "Condensed", out);
        // Command line params
        printInfoRow("Invocation Params", (String) context.get("commandLine"), out);
        out.println("			</table>");
        endModBody(out);
	}

	private String renderBreakableRegex(String pattern) {
		if (StringUtil.isEmpty(pattern))
			return pattern;
		return pattern.replace("|", "|<wbr/>");
	}

	protected void renderStats(SanityCheckSuite suite, FilePrintStream out) {
		modSeparator("Stats", out);
	    startModBody(out);
        out.println("			<table cellpadding='3'>");
        // Started
        printInfoRow("Started",  context.formatDate(suite.getStartDate()), out);
        // Duration
        printInfoRow("Duration", TimeUtil.formatDuration(suite.getExecutionTime(), true, false), out);
        out.println("			</table>");
        endModBody(out);
	}

	protected void renderCheckedTags(SanityCheckSuite suite, FilePrintStream out) {
		if (!CollectionUtil.isEmpty(context.getTags())) { // render tags only if they were restricted
			modSeparator("Checked Tags", out);
		    startModBody(out);
		    CheckTagAggregator visitor = new CheckTagAggregator(
		    		context.isUsingNamespaces(), context.isAuthorInCloud(), context.isTableInCloud()) {
				@Override
				protected long weight(SanityCheck check) {
					return 1;
				}
			};
			suite.accept(visitor);
		    Map<String, Long> weights = visitor.getWeights();
			for (String tag : weights.keySet()) {
				out.print(tag);
				out.print(" ");
			}
		}
	}

	protected void printInfoRow(String header, String value, FilePrintStream out) {
        out.println("				<tr valign='top'>" +
            "<th align='left'>" + header + ":</th>" +
            "<td align='left'>" + value + "</td>" +
            "</tr>");
	}

}
