/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import static org.databene.dbsanity.report.ReportUtil.*;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.databene.commons.FileUtil;
import org.databene.commons.NameComparator;
import org.databene.commons.file.FilePrintStream;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.CheckHolder;
import org.databene.dbsanity.model.SuiteHolder;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SanityCheckVerdict;

/**
 * Provides an overview of a DB sanity sub suite.<br/><br/>
 * Created: 08.11.2010 17:51:11
 * @since 0.4
 * @author Volker Bergmann
 * @author Yibo Wang
 */
public class SubSuiteModule extends AbstractReportModule {
	
	private boolean condensed;
	private int recursionDepth;
	
	public SubSuiteModule(int recursionDepth, boolean condensed) {
		this.condensed = condensed;
		this.recursionDepth = recursionDepth;
	}

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return 2;
	}
	
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite, FilePrintStream out) {
		try {
			FileUtil.ensureDirectoryExists(suite.getReportFolder());
			ReportUtil.copyResourceFiles(suite.getReportFolder(), style);
			renderGroupAndCheckModules(suite, recursionDepth, out);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void close() {
		super.close();
	}
	
	// private helper methods --------------------------------------------------
	
	private void renderGroupAndCheckModules(SanityCheckSuite suite, int depth, FilePrintStream out) {
		if (suite instanceof CheckHolder && containsApplicableCheck((CheckHolder) suite))
			renderCheckListModule((CheckHolder) suite, out);
		if (suite instanceof SuiteHolder && containsSuitesWithApplicableChecks((SuiteHolder) suite))
			renderGroupModule((SuiteHolder) suite, depth, out);
	}

	private boolean containsApplicableCheck(CheckHolder checkHolder) {
		for (SanityCheck check : checkHolder.getChecks())
			if (check.getVerdict() != SanityCheckVerdict.NOT_APPLICABLE)
				return true;
		return false;
	}

	public boolean containsSubSuiteHolders(SuiteHolder parent) {
		for (SuiteHolder childSuite : parent.getChildSuites(SuiteHolder.class))
			if (childSuite.countChecks() > 0)
				return true;
		return false;
	}

	protected void renderGroupModule(SuiteHolder suiteHolder, int depth, FilePrintStream out) {
		startModule(out);
		String suiteName = suiteHolder.getName();
		String title = (suiteName != null ? suiteName + " - sub groups" : "Sub Groups");
		modHeader(title, out);
		startModBody(out);
		renderSubSuitesTable(suiteHolder, out);
		if (depth != 0) {
			out.println("<br>");
			for (SanityCheckSuite childSuite : suiteHolder.getChildSuites())
				renderGroupAndCheckModules(childSuite, depth - 1, out);
		}
		endModBody(out);
		endModule(out);
	}

	protected void renderCheckListModule(CheckHolder checkHolder, FilePrintStream out) {
		startModule(out);
		String suiteName = checkHolder.getName();
		String title = (suiteName != null && !"checks".equalsIgnoreCase(suiteName) ? suiteName + " checks" : "Checks");
		modHeader(title, out);
		startModBody(out);
		renderCheckTable(checkHolder, out);
		endModBody(out);
		endModule(out);
	}

	private boolean containsSuitesWithApplicableChecks(SuiteHolder suiteHolder) {
		for (SanityCheckSuite childSuite : suiteHolder.getChildSuites())
			if (childSuite.countApplicableChecks() > 0)
				return true;
		return false;
	}

	private void renderSubSuitesTable(SuiteHolder parent, FilePrintStream out) {
	    out.println("					<table width='100%' cellpadding='3' border='0'>");
	    out.println("						<thead>");
	    out.println("							<tr>" +
	    			"								<th width='5%'  align='left'>Verdict</th>" +
	    			"								<th width='20%' align='left'>Group</th>" +
	    			"								<th width='5%'  align='right'>Defects</th>" +
	    			"								<th width='70%' align='left'>Verdicts</th>"+ 
	    			"							</tr>");
	    out.println("						</thead>");
	    out.println("						<tbody>");
	    List<SanityCheckSuite> childSuites = new ArrayList<SanityCheckSuite>(parent.getChildSuites());
	    Collections.sort(childSuites, new NameComparator());
		int maxTestCount = 0;
		for (SanityCheckSuite childSuite : childSuites)
            maxTestCount = Math.max(getTestCount(childSuite), maxTestCount);
	    for (SanityCheckSuite child : childSuites) {
    		out.println("							<tr>");
    		out.println("								<td align='center'>" + context.verdictImage(child.getVerdict(), out.getFile()) + "</td>");
			out.println("								<td align='left'>" + fileOrFolderRef(out.getFile(), child) + "</td>"); 
			out.println("								<td align='right' class='defect'>" + (child.getDefectCount()  > 0 ? context.formatLong(child.getDefectCount())  : "&nbsp;") + "</td>");
    		out.println("								<td width='75%' align='left'>");
    		summaryBar(child, (100 * getTestCount(child) / (double) maxTestCount) + "%", out);
    		out.println("								</td>");
    		out.println("							</tr>");
	    }
	    out.println("						</tbody>");
	    out.println("					</table>");
    }

	private String fileOrFolderRef(File fromFile, SanityCheckSuite toChild) {
		String childName = toChild.getName();
		String relativePath = FileUtil.relativePath(fromFile, toChild.getReportFolder());
		String childRef = relativePath + "/index.html";
		return "<a href=\"" + childRef + "\">" + childName + "</a>";
	}

	private void renderCheckTable(SanityCheckSuite child, FilePrintStream out) {
	    out.println("	<table width='100%' cellpadding='3' border='0' cellspacing='0'>");
	    out.println("		<tr>" +
	    			"			<th width='5%'  class='trow' align='center'>Verdict</th>" +
	    			"			<th width='75%' class='trow' align='left'>Check</th>" +
	    			"			<th width='10%' class='trow' align='right'>Defects</th>" +
	    			"			<th width='10%' class='trow' align='right'>Duration</th>" +
					"		</tr>");
	    for (SanityCheck check : ((CheckHolder) child).getChecks()) {
	    	SanityCheckVerdict verdict = check.getVerdict();
	    	if (verdict != SanityCheckVerdict.NOT_APPLICABLE && (!condensed || verdict == SanityCheckVerdict.FAILED || verdict == SanityCheckVerdict.ERROR)) {
		    	out.print("		<tr>");
	    		out.println("			<td align='center' class='trow'>" + context.verdictImage(verdict, out.getFile()) + "</td>");
	    		String note = (check.getExecutionNote() != null ? " (" + check.getExecutionNote() + ')' : "");
				out.println("			<td align='left' class='trow'><a href='" + docOrDefectFileRef(check, out.getFile()) + "'>" + check.getName() + note + "</a></td>");
				out.println("			<td align='right' class='trow_defect'>" 
					+ context.formatLong(check.getDefectCount(), "&nbsp;") + "</td>");
				int time = check.getExecutionTime();
				out.println("			<td align='right' class='trow'>"
					+ (time > 0 ? context.formatLong(time) + " ms" : "") + "</td>");
		    	out.println("		</tr>");
	    	}
	    }
	    out.println("	</table>");
    }

}
